@@ -25,8 +25,8 @@ pub fn show_complete(code: &str, position: Position) -> String {
 fn apply_conversion(src: &str, completions: Vec<CompletionItem>, value: &str) -> String {
     let completion = completions
         .iter()
-        .find(|c| c.label == value.to_string())
-        .expect(&format!("no completion with value `{value}`"));
+        .find(|c| c.label == *value)
+        .unwrap_or_else(|| panic!("no completion with value `{value}`"));
 
     let mut edits = vec![];
     if let Some(lsp_types::CompletionTextEdit::Edit(edit)) = &completion.text_edit {
