@@ -256,7 +256,7 @@ macro_rules! assert_warnings_with_imports {
             vec![
                 $(("thepackage", $name, $module_src)),*
             ],
-            crate::build::Target::Erlang,
+            $crate::build::Target::Erlang,
             None
         );
 
@@ -272,7 +272,7 @@ macro_rules! assert_warnings_with_imports {
 #[macro_export]
 macro_rules! assert_warning {
     ($src:expr) => {
-        let warning = $crate::type_::tests::get_printed_warnings($src, vec![], crate::build::Target::Erlang, None);
+        let warning = $crate::type_::tests::get_printed_warnings($src, vec![], $crate::build::Target::Erlang, None);
         assert!(!warning.is_empty());
         let output = format!("----- SOURCE CODE\n{}\n\n----- WARNING\n{}", $src, warning);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
@@ -309,7 +309,7 @@ macro_rules! assert_js_warning {
         let warning = $crate::type_::tests::get_printed_warnings(
             $src,
             vec![],
-            crate::build::Target::JavaScript,
+            $crate::build::Target::JavaScript,
             None,
         );
         assert!(!warning.is_empty());
@@ -324,7 +324,7 @@ macro_rules! assert_js_no_warnings {
         let warning = $crate::type_::tests::get_printed_warnings(
             $src,
             vec![],
-            crate::build::Target::JavaScript,
+            $crate::build::Target::JavaScript,
             None,
         );
         assert!(warning.is_empty());
@@ -337,7 +337,7 @@ macro_rules! assert_warnings_with_gleam_version {
         let warning = $crate::type_::tests::get_printed_warnings(
             $src,
             vec![],
-            crate::build::Target::Erlang,
+            $crate::build::Target::Erlang,
             Some($gleam_version),
         );
         assert!(!warning.is_empty());
@@ -349,7 +349,7 @@ macro_rules! assert_warnings_with_gleam_version {
 #[macro_export]
 macro_rules! assert_no_warnings {
     ($src:expr $(,)?) => {
-        let warnings = $crate::type_::tests::get_warnings($src, vec![], crate::build::Target::Erlang, None);
+        let warnings = $crate::type_::tests::get_warnings($src, vec![], $crate::build::Target::Erlang, None);
         assert_eq!(warnings, vec![]);
     };
     ($(($package:expr, $name:expr, $module_src:literal)),+, $src:expr $(,)?) => {
